program kmeans
  implicit none

  ! Number of cluster
  !integer, parameter :: CLUSTER_SIZE = 5
  integer, parameter :: CLUSTER_SIZE = 53
  ! Number of points
  !integer, parameter :: POINTS_SIZE = 100000
  integer, parameter :: POINTS_SIZE = 1000000
  ! Dimension of position
  integer, parameter :: POINTS_DIMENSION = 2
  ! Max iteration for convergence
  !integer, parameter :: MAX_ITER = 100
  ! Size of image output
  integer, parameter :: X_MAX = 640
  integer, parameter :: Y_MAX = 480
  ! Kind pour double precision
  integer, parameter :: dp = selected_real_kind(15,307)

  type :: point
    ! position of the points
    real(kind=dp), dimension(POINTS_DIMENSION) :: position
    ! centroid  id  associated
    integer :: centroid_id
  end type

  type :: centroid
    ! position of center of centroid
    real(kind=dp), dimension(POINTS_DIMENSION) :: position
    ! accumulate position (for computation of new position)
    real(kind=dp), dimension(POINTS_DIMENSION) :: acc_position
    ! accumulate number of points
    integer :: acc_points
    ! color for image output
    integer,dimension(3) :: color
  end type

  type(point) , dimension(:), allocatable :: points
  type(centroid), dimension(:), allocatable :: centroids
  integer :: iter,iterk,ufile,iterc,row,col,nbiter
  real :: alea
  real :: start,finish
  logical :: again
  real(kind=dp) :: distmin, dist
  integer :: idmin
  integer, dimension(:,:,:), allocatable :: img

  ! Random init points
  allocate(points(POINTS_SIZE))
  do iter=1, POINTS_SIZE
    call random_number(points(iter)%position(:))
    points(iter)%centroid_id = -1
  end do

  ! Init centroid (choose randomly points for centroids)
  allocate(centroids(CLUSTER_SIZE))
  do iter=1, CLUSTER_SIZE
    ! Choose randomly a point
    call random_number(alea)
    centroids(iter)%position(:) = points(1+int(alea*(POINTS_SIZE-1)))%position(:)
    ! Define randomly a centroid color
    do iterc=1,3
      call random_number(alea)
      centroids(iter)%color(iterc) = (int(alea*256)+125)/2
    end do
  end do

  call cpu_time(start)
  nbiter = 0
  again = .true.
  do while(again)
    nbiter=nbiter+1
    again = .false.

    ! Init centroid
    do iterk=1, CLUSTER_SIZE
      centroids(iterk)%acc_position(1) = 0._dp
      centroids(iterk)%acc_position(2) = 0._dp
      centroids(iterk)%acc_points = 0
    end do

    ! Find nearest centroid for all points
    do iter=1,POINTS_SIZE
      ! Starting with the last centroid
      distmin = (centroids(CLUSTER_SIZE)%position(1)-points(iter)%position(1))**2 + &
      (centroids(CLUSTER_SIZE)%position(2)-points(iter)%position(2))**2
      idmin = CLUSTER_SIZE
      ! Main loop
      do iterk=1, CLUSTER_SIZE-1
        dist = (centroids(iterk)%position(1)-points(iter)%position(1))**2 + &
        (centroids(iterk)%position(2)-points(iter)%position(2))**2
        if (dist < distmin) then
          distmin = dist
          idmin = iterk
        end if
      end do

      ! This point has a new nearest centroid ?
      if (idmin /= points(iter)%centroid_id ) then
        ! Yes so we do the while loop again
        points(iter)%centroid_id = idmin
        again = .true.
      end if

      ! Add point for gravity center computation of centroids
      centroids(idmin)%acc_position(1) = centroids(idmin)%acc_position(1)+ &
        points(iter)%position(1)
      centroids(idmin)%acc_position(2) = centroids(idmin)%acc_position(2)+ &
        points(iter)%position(2)
      centroids(idmin)%acc_points = centroids(idmin)%acc_points+1
    end do

    ! Position of centroids = gravity center of nearest points
    do iterk=1,CLUSTER_SIZE
      if (centroids(iterk)%acc_points /= 0) then
        centroids(iterk)%position(1) = centroids(iterk)%acc_position(1)/ &
         centroids(iterk)%acc_points
        centroids(iterk)%position(2) = centroids(iterk)%acc_position(2)/ &
         centroids(iterk)%acc_points
      end if
    end do
  end do
  call cpu_time(finish)
  print *, "Time ", finish-start,"s", nbiter,"its", nbiter/(finish-start),"it/s"

  ! Print position of centroids
  do iterk=1, CLUSTER_SIZE
    print *, iterk, centroids(iterk)%position(:)
  end do

  ! Write image
  allocate(img(3,X_MAX,Y_MAX))
  img = 0
  do iter=1,POINTS_SIZE
    col = int((X_MAX-1)*points(iter)%position(1))+1
    row = int((Y_MAX-1)*points(iter)%position(2))+1
    do iterc=1,3
      img(iterc,col,row) = centroids(points(iter)%centroid_id)%color(iterc)
    end do
  end do
  do iterk=1,CLUSTER_SIZE
    col = int((X_MAX-1)*centroids(iterk)%position(1))+1
    row = int((Y_MAX-1)*centroids(iterk)%position(2))+1
    if ((col > 0) .and. (col < X_MAX) .and. (row > 0) .and. (row < Y_MAX) ) then
      do iterc=1,3
        img(iterc,col,row) = 255
      end do
    end if
  end do
  open(newunit=ufile,file="out.ppm")
  write(ufile,'(A2)') 'P6'
  write(ufile,'(i0," ",i0)') X_MAX,Y_MAX
  write(ufile,'(i0)') 255
  do row=1,Y_MAX
    do col=1,X_MAX
      write(ufile, '(3A1)', advance='no') img(1,col,row),img(2,col,row), &
       img(3,col,row)
    end do
  end do
  close(ufile)

end program
