program heatnum
  implicit none

  integer :: nx, ny
  double precision,dimension(:,:,:), allocatable :: v
  integer :: iterx, itery, new, old,iter
  double precision :: diff, delta,inv9
  integer :: lsx,lex

  nx = 500
  ny = 300

  call align2d(1,nx,2,8,lsx,lex)
  allocate(v(lsx:lex,ny,2))
  v(:,:,:) = 0.d0
  do iterx=1, nx
    v(iterx,1,1) = iterx 
    v(iterx,1,2) = iterx
    v(iterx,ny,1) = iterx+(ny-1)*nx
    v(iterx,ny,2) = iterx+(ny-1)*nx
  end do
  do itery=1,ny
    v(1,itery,1) = 1+(itery-1)*nx
    v(1,itery,2) = 1+(itery-1)*nx
    v(nx,itery,1) = nx+(itery-1)*nx
    v(nx,itery,2) = nx+(itery-1)*nx
  end do

  new = 2
  old = 1
  diff = 1000000.d0
  iter = 0
  inv9 = 1/9.d0

  do while (diff > 1e-7)
    diff = 0.d0
    do itery=2, ny-1
      !$omp simd private(delta) reduction(max:diff)
      do iterx=2, nx-1
        v(iterx,itery,new) = (v(iterx-1,itery+1,old)+v(iterx,itery+1,old)+v(iterx+1,itery+1,old)+ &
                              v(iterx-1,itery,old)+v(iterx,itery,old)+v(iterx+1,itery,old)+ &
                              v(iterx-1,itery-1,old)+v(iterx,itery-1,old)+v(iterx+1,itery-1,old))*inv9
        delta = abs(v(iterx,itery,new)-v(iterx,itery,old))
        if (delta > diff) diff = delta
      end do
    end do
    iter = iter+1
    old = new
    new = 2-mod(old+1,2)
  end do

  print *, iter, diff, v(nx/2, ny/2, old), nx/2+(ny/2-1)*nx
  contains
    subroutine align1d(bsx,bex,isx,mult,sx)
      integer, intent(in) :: bsx ! Original start bound
      integer, intent(in) :: bex ! Original end bound
      integer, intent(in) :: isx ! Start index computation
      integer, intent(in) :: mult ! Multiple to align
      integer, intent(out) :: sx ! New start bound
      integer :: temp

      temp = isx-bsx
      if (mod(temp,mult) == 0) then
        ! Already aligned
        sx = bsx
      else
        sx = bsx-mult+mod(temp,mult)
      end if
    end subroutine align1d

    subroutine align2d(bsx,bex,isx,mult,sx,ex)
      integer, intent(in) :: bsx ! Original start bound
      integer, intent(in) :: bex ! Original end bound
      integer, intent(in) :: isx ! Start index computation
      integer, intent(in) :: mult ! Multiple to align
      integer, intent(out) :: sx ! New start bound
      integer, intent(out) :: ex ! New end bound
      integer :: temp

      call align1d(bsx,bex,isx,mult,sx)
      temp = bex-sx+1
      if (mod(temp,mult) == 0) then
        ex = bex
      else
        ex = bex+mult-mod(temp,mult)
      end if
      ! avoid size = 2^x
      temp = ex-sx+1
      if (iand(temp,temp-1) == 0) then
        ex=ex+mult
      end if
    end subroutine align2d
end program
